/*FILE:
*******************************************************************************
**
**  $Workfile: VOSSFMGR.HPP $
**
**  $Revision: 11 $
**
**  $Date: 8/23/94 9:33p $
**
**  $Author: David $
**
**  Description:
**
**  Contents:
**
**  Notes:
**
**  See Also:
**
**  Copyright (C) 1994, Van de Vate Object-Oriented Software Solutions
**
*******************************************************************************
*/

#ifndef VOSSFMGR_HPP
#define VOSSFMGR_HPP

/******************************************************************************
**  Include Standard C/C++ Header Files (windows.h last)
*******************************************************************************
*/

#include <windows.h>

/******************************************************************************
**  Include Project Specific Header Files
*******************************************************************************
*/

/******************************************************************************
**  Include Module Specific Header Files
*******************************************************************************
*/


/******************************************************************************
**  Define Public Constant Macros (#define)
*******************************************************************************
*/

#define VFMEVENT_LOAD            100

/******************************************************************************
**  Define Public enums
*******************************************************************************
*/

/******************************************************************************
**  Define Public typedefs
*******************************************************************************
*/
class _export VFileManagerExtension;

/*CLASS:
--------------------------------------------------------------------------------
**  Name : VFMXTime
**
**  Base : None
**
**  Description : This class encapsulates a time of day.
**
**  See Also :
**
--------------------------------------------------------------------------------
*/
class _export VFMXTime
{
   private:
      WORD wHour, wMin, wSec;
   public:
      VFMXTime(void) {};
      VFMXTime(UINT wFileTime);

      WORD Hour(void)   const {return wHour;}
      WORD Minute(void) const {return wMin;}
      WORD Second(void) const {return wSec;}
};

/*CLASS:
--------------------------------------------------------------------------------
**  Name : VFMXDate
**
**  Base : None
**
**  Description : This class encapsulates the date.
**
**  See Also :
**
--------------------------------------------------------------------------------
*/
class _export VFMXDate
{
   private:
      WORD wMonth,wDay,wYear;
   public:
      VFMXDate(void) {};
      VFMXDate(UINT wFileDate);

      WORD Month(void)  const {return wMonth;}
      WORD Day(void)    const {return wDay;}
      WORD Year(void)   const {return wYear;}

};



//CLASS:
//**************************************************************************
//  Name : VFMXFileInfo
//
//  Base : None
//
//  Description :  This class provides a more intuitive access to the
//                 File Manager Extension data structure FMS_GETFILESEL.
//
//
//**************************************************************************
class VFMXFileInfoInt;
class VFileManagerExtensionInt;
class _export VFMXFileInfo
{
   friend VFileManagerExtensionInt;
   private:
      VFMXFileInfoInt& oInfo;

      VFMXFileInfo(void);
      virtual ~VFMXFileInfo(void);

   public:
      enum VFMXAttribState {test=0,set=1,clear=2};

      const char *Name(void) const;

      const VFMXTime& Time(void);
      const VFMXDate& Date(void);

      DWORD  Size(void)  const;

      //
      //  file attribute access methods
      //
      BOOL   ReadOnly   (VFMXAttribState etState=test);
      BOOL   Hidden     (VFMXAttribState etState=test);
      BOOL   System     (VFMXAttribState etState=test);
      BOOL   VolumeLabel(VFMXAttribState etState=test);
      BOOL   Directory  (VFMXAttribState etState=test);
      BOOL   Archive    (VFMXAttribState etState=test);
      BOOL   Normal     (VFMXAttribState etState=test);

      //
      //  internal data structure
      //

      operator VFMXFileInfoInt&();
};

//CLASS:
//**************************************************************************
//  Name : VFMXDriveInfo
//
//  Base : None
//
//  Description : This class provides a more intuitive interface to the
//                File Manager Extension data structure FMS_GETDRIVEINFO.
//
//  See Also :
//
//**************************************************************************
class VFMXDriveInfoInt;
class _export VFMXDriveInfo
{
   friend VFileManagerExtensionInt;
   private:
      VFMXDriveInfoInt& oInfo;


      VFMXDriveInfo(void);
      virtual ~VFMXDriveInfo(void);
   public:

      DWORD TotalSpace(void)                const;
      DWORD FreeSpace(void)                 const;
      const char *CurrentDirectory(void)    const;
      const char *VolumeLabel(void)         const;
      const char *SharePoint(void)          const;

      //
      //  internal data structure
      //

      operator VFMXDriveInfoInt&();
};

//CLASS:
//**************************************************************************
//  Name : VFMXMenuItem
//
//  Base : None
//
//  Description : This class is a base class from which menu item behavior
//                may be defined.
//
//  See Also :
//
//**************************************************************************

class _export VFileManagerExtension;
class _export VFMXMenuItem
{
   friend VFMXMenuItemInt;
   private:
      int                     iMenuCommand;
      VFileManagerExtension *opExtension;
   protected:
   public:
      //VFMXMenuItem(void)  {opExtension=NULL,iMenuCommand=0;}
      VFMXMenuItem(VFileManagerExtension *opExtension,int iMenuCommand);

      //
      //  method to obtain extension class.
      //
      VFileManagerExtension * Extension(void) {return opExtension;}

      //
      //  methods to determine attributes of menu items.
      //
      virtual int          IsEnabled(void)       {return 1;}
      virtual int          IsChecked(void)       {return 0;}
      virtual HBITMAP      CheckedBitmap(void)   {return 0;}
      virtual HBITMAP      UncheckedBitmap(void) {return 0;}
      virtual void         Execute(void)         {}
      virtual const char * HelpString(void)      {return 0;}
      virtual void         ShowHelp(void)        {};

      void CheckMarkDimensions(int& iWidth,int& iHeight);

      int MenuCommand(void) const {return iMenuCommand;}

      int operator==(const VFMXMenuItem& oItem) const
                    {return MenuCommand() == oItem.MenuCommand();}

      int operator< (const VFMXMenuItem& oItem) const
                    {return MenuCommand() <  oItem.MenuCommand();}
};


//CLASS:
//**************************************************************************
//  Name : VFileManagerExtension
//
//  Base : None
//
//  Description : This class is a base class from which File Manager
//                extension behavior can be derived.
//
//  See Also :
//
//**************************************************************************

class _export VFileManagerExtension
{
   private:
      VFileManagerExtensionInt& oInfo;
   protected:
      BOOL  Win32Version(void) const;

   public:
      enum VFMWindowTypeET {Directory=1,Tree=2,Drives=3,Search=4};

      VFileManagerExtension(void);
      virtual ~VFileManagerExtension(void);

      //
      //  Information that may be obtained from the File Manager.
      //
      WORD            Delta(void)        const;
      HWND            FileManager(void)  const;
      VFMXDriveInfo&  DriveInfo(void);
      VFMXFileInfo&   FileSelection(int iSelectionIdx=0);
      VFMXFileInfo&   operator[](int iSelectionIdx)
                            {return FileSelection(iSelectionIdx);}
      WORD            SelectionCount(void);
      VFMWindowTypeET WindowTypeInFocus(void);


      //
      //  Commands that may be directed to the File Manager.
      //
      void Refresh(BOOL bAllWindows=FALSE);
      void Reload(void);

      //
      //  Messages received from the File Manager.
      //
      virtual void InitMenu(HMENU hPopupMenu,int iMenuDelta);
      virtual void SelectionChange(void);
      virtual void UserRefresh(void);

      //
      //  Information required of derived extensions.
      //
      virtual HMENU       ExtensionMenu(void) = 0;
      virtual const char *MenuName(void)      = 0;

      //
      //  Called for Menu Commands
      //
      virtual void MenuCommand(WORD wItemNumber,BOOL bHelpWanted=FALSE);

      //
      //  Get Help String
      //
      virtual const char * ItemHelpString(WORD wItemNumber);

      //
      //  Called for all messages
      //
      virtual void Process(WORD wMsg,LPARAM lParam);

      //
      //  method to return internal data structure
      //
      operator VFileManagerExtensionInt&(void)  {return oInfo;}

      //
      //  user registeration string
      //
      virtual const char *UserRegistration(void) const  {return NULL;}
};



/******************************************************************************
**  Define Public Inline Function Macros
*******************************************************************************
*/

/******************************************************************************
**  Define Public Global Variables (extern only)
*******************************************************************************
*/

#ifndef _VOSSMODNAME
VFileManagerExtension * gopExtension = NULL;
#endif

/******************************************************************************
**  Declare Public Function Prototypes (Listed in Alphabetical Order)
*******************************************************************************
*/
extern "C" LONG WINAPI _export VFMExtensionProc(VFileManagerExtension *&,
                                           HWND,WORD,LONG);

extern "C" LONG WINAPI _export FMExtensionProc(HWND hwnd, WORD wEvent, LONG lParam);


#ifndef _VOSSMODNAME
VFileManagerExtension * AllocateExtension(void);

extern "C"  LONG WINAPI FMExtensionProc(HWND hFileMgr,WORD wMsg,
                                                   LONG lParam)
{
   if (wMsg == VFMEVENT_LOAD)
       gopExtension = AllocateExtension();

   return VFMExtensionProc(gopExtension,hFileMgr,wMsg,lParam);
}

#endif


#endif



/******************************************************************************
**                          E N D   O F   C O D E
*******************************************************************************
*/

/******************************************************************************
**                   R E V I S I O N  I N F O R M A T I O N
*******************************************************************************
**
** $History: VOSSFMGR.HPP $
**
*******************  Version 11  *****************
**User: David        Date: 8/23/94    Time: 9:33p
**Branched in $/FILEMANAGER/Release 1.5/INCLUDE
**
*******************  Version 10  *****************
**User: David        Date: 8/23/94    Time: 9:32p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**
*******************  Version 9  *****************
**User: David        Date: 8/21/94    Time: 11:31p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**Added help API to VFMXMenuItem
**
*******************  Version 8  *****************
**User: David        Date: 8/20/94    Time: 11:27p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**Added VFMXMenuItem processing
**
*******************  Version 7  *****************
**User: David        Date: 8/17/94    Time: 10:22p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**
*******************  Version 6  *****************
**User: David        Date: 8/10/94    Time: 11:07p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**Modified dos file attribute access methods.
**
*******************  Version 5  *****************
**User: David        Date: 8/02/94    Time: 11:30p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**Added migration path to  support for WIN32 features.
**Made VFileManagerExtensionInt class.
**
*******************  Version 4  *****************
**User: David        Date: 8/01/94    Time: 10:47p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**Added comments.
**
*******************  Version 3  *****************
**User: David        Date: 7/31/94    Time: 11:12p
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**
*******************  Version 2  *****************
**User: David        Date: 7/31/94    Time: 12:07a
**Updated in $/TOOLS/VOSSFMGR/INCLUDE
**
*******************  Version 1  *****************
**User: David        Date: 7/30/94    Time: 12:00a
**Created in $/TOOLS/VOSSFMGR/INCLUDE
**
*******************************************************************************
*/



/******************************************************************************
**                          E N D   O F   FILE
*******************************************************************************
*/
